(define-module (ordo connection ssh)
  #:use-module (oop goops)
  #:use-module (ice-9 exceptions)
  #:use-module (ice-9 popen)
  #:use-module (ssh session)
  #:use-module (ssh channel)
  #:use-module (ssh auth)
  #:use-module (ssh popen)
  #:use-module (ssh sftp)
  #:use-module (ordo connection base)
  #:use-module (ordo util read-lines)
  #:export (<ssh-connection>))

(define-class <ssh-connection> (<connection>)
  (user #:getter user #:init-keyword #:user)
  (host #:getter host #:init-keyword #:host)
  (password #:getter password #:init-keyword #:password #:init-val #f)
  (identity #:getter identity #:init-keyword #:identity #:init-val #f)
  (authenticate-server? #:getter authenticate-server? #:init-keyword #:authenticate-server? #:init-val #t)
  (session #:accessor session)
  (sftp-session #:accessor sftp-session))

(define-method (conn:setup (c <ssh-connection>))
  (unless (slot-bound? c 'session)
    (set! (session c) (make-session #:user (user c) #:host (host c)))
    (when (identity c) (session-set! (session c) 'identity (identity c))))
  (let ((s (session c)))
    (unless (connected? s)
      (connect! s)
      (when (authenticate-server? s)
        (let ((server-auth (authenticate-server s)))
          (unless (equal? 'ok server-auth)
            (error (format #f "authenticate-server: ~a" server-auth)))))
      (let ((user-auth (if (password c)
                           (userauth-password! s (password c))
                           (userauth-public-key/auto! s))))
        (unless (equal? 'success user-auth)
          (error (format #f "userauth: ~a" user-auth)))))))

(define-method (conn:run (c <ssh-connection>) (command <string>))
  (let* ((channel (open-remote-input-pipe (session c) command))
         (output (read-lines channel))
         (exit-status (channel-get-exit-status channel)))
    (close channel)
    (values output exit-status)))

(define-method (sftp-session (c <ssh-connection>))
  (unless (slot-bound? c 'sftp-session)
    (set! (sftp-session c) (make-sftp-session (session c))))
  (sftp-session c))

(define-method (conn:call-with-input-file (c <ssh-connection>) (filename <string>) (proc <procedure>))
  (call-with-remote-input-file (sftp-session c) filename proc))

(define-method (conn:call-with-output-file (c <ssh-connection>) (filename <string>) (proc <procedure>))
  (call-with-remote-output-file (sftp-session c) filename proc))

(define-method (conn:teardown (c <ssh-connection>))
  (when (slot-bound? c 'session)
    (let ((s (session c)))
      (when (connected? s)
        (disconnect! s)))))
