(define-module (ordo connection sudo)
  #:use-module (oop goops)
  #:use-module (ice-9 exceptions)
  #:use-module (ice-9 rdelim)
  #:use-module (srfi srfi-26)
  #:use-module (srfi srfi-71)
  #:use-module (ordo connection base)
  #:use-module (ordo util shell-quote)
  #:export (<sudo-connection>))

(define-class <sudo-connection> (<connection>)
  (connection #:getter connection #:init-keyword #:connection)
  (become-user #:getter become-user #:init-keyword #:become-user #:init-form #f)
  (become-password #:getter become-password #:init-keyword #:become-password #:init-form #f)
  (password-tmp-file #:accessor password-tmp-file))

(define-method (conn:validate (c <sudo-connection>))
  (conn:validate (connection c)))

(define-method (conn:setup (c <sudo-connection>))
  (conn:setup (connection c))
  (when (become-password c)
    (let ((out rc (conn:run (connection c) "mktemp")))
      (unless (zero? rc)
        (raise-exception (make-exception
                          (make-external-error)
                          (make-exception-with-message (format #f "Failed to create temporary directory: ~a" (car out))))))
      (let ((tmp-file (car out)))
        (conn:call-with-output-file (connection c) tmp-file (cut write-line (become-password c) <>))
        (set! (password-tmp-file c) tmp-file)))))

(define-method (sudo-command (c <sudo-connection>))
  (cond
   ((and (become-user c) (become-password c))
    (format #f "cat ~a - | sudo -k -S -H -u ~a" (string-shell-quote (password-tmp-file c)) (string-shell-quote (become-user c))))

   ((become-password c)
    (format #f "cat ~a - | sudo -k -S -H" (string-shell-quote (password-tmp-file c))))

   ((become-user c)
    (format #f "sudo -k -n -H -u ~a" (string-shell-quote (become-user c))))

   (else "sudo -k -n -H")))

(define-method (conn:teardown (c <sudo-connection>))
  (when (slot-bound? c 'password-tmp-file)
    (conn:run (connection c) (format #f "rm -f ~a" (string-shell-quote (password-tmp-file c)))))
  (conn:teardown (connection c)))

(define-method (conn:run (c <sudo-connection>) (command <string>))
  (let ((command (string-append (sudo-command c) " -- " command)))
    (conn:run (connection c) command)))

;; There is no special sudo handling for file I/O. This means the caller needs to
;; ensure that they have read/write access to the target file.
(define-method (conn:call-with-input-file (c <sudo-connection>) (filename <string>) (proc <procedure>))
  (conn:call-with-input-file (connection c) filename proc))

(define-method (conn:call-with-output-file (c <sudo-connection>) (filename <string>) (proc <procedure>))
  (conn:call-with-output-file (connection c) filename proc))
