(define-module (ordo interceptor user-info)
  #:use-module (rx irregex)
  #:use-module (srfi srfi-1)
  #:use-module (srfi srfi-145)
  #:use-module (ordo connection)
  #:use-module (ordo interceptor)
  #:use-module (ordo util shell-quote)
  #:export (user-info))

(define (parse-id s)
  (let ((data (reverse (irregex-fold (irregex '(seq (=> id integer) "(" (=> name (+ alphanumeric)) ")"))
                                     (lambda (_ m accum)
                                       (cons `((#:id . ,(string->number (irregex-match-substring m 'id)))
                                               (#:name . ,(irregex-match-substring m 'name)))
                                             accum))
                                     '()
                                     s))))
    `((#:user-id . ,(assoc-ref (first data) #:id))
      (#:user-name . ,(assoc-ref (first data) #:name))
      (#:group-id . ,(assoc-ref (second data) #:id))
      (#:group-name . ,(assoc-ref (second data) #:name))
      (#:groups . ,(drop data 2)))))

(define (parse-passwd-entry s)
  (map cons
       '(#:user-name #:password #:user-id #:group-id #:gecos #:home-dir #:shell)
       (string-split s #\:)))

(define* (user-info #:key (register 'user-info))
  (assume (symbol? register) "register should be a symbol" register)
  (interceptor
   "user-info"
   #:enter (lambda (ctx)
             (let* ((conn (context-connection ctx))
                    (id (run conn "id"
                             #:check? #t #:return (compose parse-id car)))
                    (pwent (run conn "getent" "passwd" (string-shell-quote (assoc-ref id #:user-name))
                                #:check? #t #:return (compose parse-passwd-entry car))))
               (var-set! ctx register (fold (lambda (key alist)
                                              (acons key (assoc-ref pwent key) alist))
                                            id
                                            (list #:gecos #:home-dir #:shell)))))
   #:leave (lambda (ctx) (var-delete! ctx register))
   #:error (lambda (ctx) (var-delete! ctx register))))
